package net.zenrindatacom.geofenceapp_sample.activity.check;

import android.util.Log;

import androidx.annotation.StringRes;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.activity.EdgeToEdgeCompatActivity;
import net.zenrindatacom.geofenceapp_sample.activity.general.CustomAsyncTask;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperManager;
import net.zenrindatacom.geofenceapp_sample.preference.GeoFenceLibPreference;
import net.zenrindatacom.geofencelib.GeoFenceManager;
import net.zenrindatacom.geofencelib.auth.AuthException;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 「エリア内外判定を実施するActivity」のベースクラス
 */
public abstract class AbstractCheckActivity extends EdgeToEdgeCompatActivity {

    private static final String TAG = "AbstractCheckActivity";

    /** 内外判定用ロックオブジェクト */
    final private Object mCheckLocationLock = new Object();

    /**
     * エリアの内外判定
     * 　ジオフェンスライブラリでエリアの内外判定を実施する
     *
     * @param latitude 緯度
     * @param longitude　経度
     * @param accuracy　測位誤差
     * @param date　測位日時
     */
    public void checkArea(double latitude, double longitude, float accuracy, Date date) {

        //ジオフェンスライブラリの認証情報
        GeoFenceLibPreference pref = GeoFenceLibPreference.getInstance();
        String apiKey = pref.getApiKey();
        String clientId = pref.getClientId();
        String secret = pref.getSecret();
        GeoFenceManager.AuthEnv authEnv = pref.getAuthEnv();

        //非同期（別スレッド）でジオフェンスライブラリのcheckLocation(内外判定)を実施する
        // →　checkLocationは内部でサーバ通信を行っているため、実行時には別スレッド化する必要がある
        new CustomAsyncTask() {
            /**
             * ジオフェンスライブラリの内外判定を実施する（synchronizedによる排他）
             */
            private List<?> checkLocation(String apiKey, String clientId, String secret, GeoFenceManager.AuthEnv authEnv, double latitude, double longitude, float accuracy, Date date) throws AuthException {
                Log.d(TAG, "checkLocation() called");
                synchronized (mCheckLocationLock) {
                    return GeoFenceWrapperManager
                            .getGeoFenceManager()
                            .checkLocation(apiKey, clientId, secret, authEnv, latitude, longitude, accuracy, date);
                }
            }

            /**
             * 非同期で内外判定を実施
             *
             * @return 内外判定の該当情報
             * @throws AuthException 認証エラー
             */
            @Override
            public List<?> doAsyncTask() throws AuthException {
                //内外判定実行
                return checkLocation(apiKey, clientId, secret, authEnv, latitude, longitude, accuracy, date);
            }

            /**
             * 非同期の処理結果通知（正常）
             * @param result 処理結果
             */
            @Override
            public void onSuccess(Object result) {

                List<?> resultList = (List<?>)result;

                StringBuilder resultString = new StringBuilder();

                if (resultList != null) {
                    for (Object obj : resultList) {
                        HashMap<String, Object> item;
                        if (obj instanceof HashMap<?, ?>) {
                            //noinspection unchecked
                            item = (HashMap<String, Object>) obj;
                            //該当したエリア情報
                            @SuppressWarnings("unchecked")
                            HashMap<String, Object> areaInfo = (HashMap<String, Object>) item.get(GeoFenceManager.RESPONSE_TAG_KEY_AREA);
                            String areaID = null;
                            if (areaInfo != null) {
                                areaID = (String) areaInfo.get(GeoFenceManager.RESPONSE_KEY_AID);
                            }

                            //該当した通知条件
                            @SuppressWarnings("unchecked")
                            HashMap<String, Object> conditionInfo = (HashMap<String, Object>) item.get(GeoFenceManager.RESPONSE_TAG_KEY_CONDITION);
                            String notificationID = null;
                            String strInOut = null;
                            if (conditionInfo != null) {
                                //通知条件ID
                                notificationID = String.valueOf(conditionInfo.get(GeoFenceManager.RESPONSE_KEY_NID));
                                //通知タイミング
                                @SuppressWarnings("ConstantConditions")
                                int inOut = (int) conditionInfo.get(GeoFenceManager.RESPONSE_KEY_NTMG);
                                strInOut = inOut == 1 ? getString(R.string.common_check_label_result_lib_in) : getString(R.string.common_check_label_result_lib_out); //1:IN, 2:OUT
                            }

                            resultString.append(String.format(getString(R.string.common_check_label_result_lib), notificationID, areaID, strInOut));
                        }
                    }
                }

                //内外判定結果を呼び元に通知
                onResult(latitude, longitude, accuracy, date, resultString.toString());
            }

            /**
             * 非同期の処理結果通知（異常）
             * @param e 認証例外
             */
            @Override
            public void onException(Exception e) {

                AuthException ex = (AuthException)e;

                @StringRes int messageResId;
                switch(ex.getErrorType()) {
                    case GET_TOKEN_AUTH_FAILED:
                        //トークン発行認証失敗
                        messageResId = R.string.toast_geofence_lib_auth_error_state_token_issue;
                        break;
                    case AUTH_NOT_AUTHORIZED:
                        //認証不許可
                        messageResId = R.string.toast_geofence_lib_auth_error_state_not_authorized;
                        break;
                    default:
                        messageResId = R.string.toast_geofence_lib_auth_error_unknown;
                        break;
                }

                //エラーメッセージを呼び元に通知
                onError(messageResId);
            }
        }.execute();
    }

    /**
     * 呼び元にジオフェンスライブラリでの例外発生を通知
     * @param messageResId メッセージID
     */
    abstract void onError(@StringRes int messageResId);

    /**
     * 呼び元にジオフェンスライブラリの内外判定結果を通知
     *
     * @param latitude 緯度
     * @param longitude 経度
     * @param accuracy 測定誤差
     * @param date 測位日時
     * @param checkResult 内外判定結果
     */
    abstract void onResult(double latitude, double longitude, float accuracy, Date date, String checkResult);
}
