package net.zenrindatacom.geofenceapp_sample.activity.check;

import android.Manifest.permission;
import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.StringRes;
import androidx.core.content.PermissionChecker;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.activity.general.DialogUtil;
import net.zenrindatacom.geofenceapp_sample.application.ILocationNotifyListener;
import net.zenrindatacom.geofenceapp_sample.application.SampleApplication;
import net.zenrindatacom.geofenceapp_sample.geofence.GeoFenceWrapperManager;
import net.zenrindatacom.geofenceapp_sample.locationlib.LocationLibConst;
import net.zenrindatacom.geofenceapp_sample.locationlib.LocationServiceManager;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

/**
 * 位置測位ライブラリで確認画面
 *
 * @author ZENRIN DataCom CO., LTD. All Rights Reserved.
 */
public class LocationLibCheckActivity extends AbstractCheckActivity implements OnClickListener {

    /** パーミッションチェック：測位開始 */
    private static final int PERMISSIONS_REQUEST_START_LOCATION = 1;
    /** 表示エリア：ライブラリへのパラメータ（位置測位の結果） */
    private TextView mLocationResultTextView;
    /** 表示エリア：結果表示（内外判定結果）*/
    private TextView mInOutResultTextView;

    /**
     * Activityの生成
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.location_lib_check);

        mLocationResultTextView = findViewById(R.id.textViewLocationResult);
        mInOutResultTextView = findViewById(R.id.textViewInOutResult);

        //位置測位結果の通知を受け取るリスナーを設定
        SampleApplication application = (SampleApplication) getApplication();
        application.setLocationNotifyListener(mLocationNotifyListener);
    }

    /**
     * Activityの破棄
     */
    @Override
    protected void onDestroy() {
        //位置測位結果の通知を受け取るリスナーを解除
        SampleApplication application = (SampleApplication) getApplication();
        application.setLocationNotifyListener(null);

        super.onDestroy();
    }

    /**
     * オプションメニューの生成
     *
     * @param menu メニュー
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.location_lib_check, menu);
        return super.onCreateOptionsMenu(menu);
    }

    /**
     * メニュー内のボタンがタップされた
     *
     * @param item タップされたメニューアイテム
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int itemId = item.getItemId();

        if(itemId == R.id.btnClear) {
            //クリア
            mLocationResultTextView.setText("");
            mInOutResultTextView.setText("");
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * 画面内のボタンがタップされた
     *
     * @param view タップされたアイテム
     */
    @Override
    public void onClick(View view) {

        //2度押し防止
        Button button = (Button)view;
        button.setEnabled(false);
        new Handler(Looper.getMainLooper()).postDelayed(() -> button.setEnabled(true), 1000);

        int viewId = view.getId();
        if (viewId == R.id.btnStart) {
            //測位開始
            if (!checkSelfPermission(this, PERMISSIONS_REQUEST_START_LOCATION)) {
                // 不許可の場合は処理終了
                return;
            }

            LocationServiceManager.startLocationService();

        } else if (viewId == R.id.btnStop) {
            //測位停止
            LocationServiceManager.stopLocationService();

        } else if (viewId == R.id.btnResetStatus) {
            //状態リセット
            GeoFenceWrapperManager.getGeoFenceManager().resetGeoFenceStatus();
            DialogUtil.showAlert(this, R.string.dlg_title_alert_geofence_lib, R.string.dlg_msg_alert_geofence_lib_reset_area_stay_and_notification_status);
        }
    }

    /**
     * パーミッションの確認
     */
    private boolean checkSelfPermission(Context context, @SuppressWarnings("SameParameterValue") int requestCode) {

        // 権限許可状況チェック
        ArrayList<String> permissions = new ArrayList<>();
        // 位置測位許可
        if (PermissionChecker.PERMISSION_GRANTED
                != PermissionChecker.checkSelfPermission(context, permission.ACCESS_FINE_LOCATION)) {
            permissions.add(permission.ACCESS_FINE_LOCATION);
        }

        // 通知許可（フォアグラウンドサービスの通知ドロワーへの表示）
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            if (requestCode == PERMISSIONS_REQUEST_START_LOCATION) {
                //連続測位の場合
                if (PermissionChecker.PERMISSION_GRANTED
                        != PermissionChecker.checkSelfPermission(this, permission.POST_NOTIFICATIONS)) {
                    Toast.makeText(getApplication(), "通知許可で許可されなかった", Toast.LENGTH_SHORT).show();
                    permissions.add(permission.POST_NOTIFICATIONS);
                }
            }
        }


        // 不許可のPermissionが存在する場合はシステムへ許可をリクエストする
        if (permissions.size() > 0) {
            requestPermissions(permissions.toArray(new String[0]), requestCode);
            return false;
        }
        return true;
    }

    /**
     * パーミッションリクエストの結果
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {

        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (PERMISSIONS_REQUEST_START_LOCATION != requestCode) {
            return;
        }

        /*
         * grantResultsにユーザーが選択した結果が返ってくる。
         * ただし、アプリが互換モードで動作している場合は必ず 「0:許可されている」が返却される。
         * そのため、grantResults を判定には使用せず、support-v4ライブラリのメソッドで判定を行っている。
         *
         * ※互換モードでなければgrantResultsは正常に値を返すため使用することはできる。
         */
        // 位置測位許可
        if (PermissionChecker.PERMISSION_GRANTED
                != PermissionChecker.checkSelfPermission(this, permission.ACCESS_FINE_LOCATION)) {
            Toast.makeText(getApplication(), R.string.toast_setting_error_permission, Toast.LENGTH_SHORT).show();
            return;
        }

        // 通知許可（フォアグラウンドサービスの通知ドロワーへの表示）
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            if (PermissionChecker.PERMISSION_GRANTED
                    != PermissionChecker.checkSelfPermission(this, permission.POST_NOTIFICATIONS)) {
                //通知許可されなくても動作するため、エラーにしないで処理続行
                Toast.makeText(getApplication(), R.string.toast_setting_info_notification_permission, Toast.LENGTH_SHORT).show();
            }
        }

        // ここまできたら権限が許可されている
        // 測位開始
        LocationServiceManager.startLocationService();
    }

    /**
     * 位置測位の結果通知を受け取るリスナー
     */
    private final ILocationNotifyListener mLocationNotifyListener = new ILocationNotifyListener() {
        /**
         * 「測位結果（OKパターン）」通知（連続測位）
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedLocationOK(Intent intent) {

            StringBuffer strBuffer = new StringBuffer();

            //プロバイダー
            addStringToBuffer(strBuffer, LocationLibConst.KEY_PROVIDER, intent.getStringExtra(LocationLibConst.KEY_PROVIDER));

            //緯度
            double latitude = intent.getDoubleExtra(LocationLibConst.KEY_LATITUDE, Double.MIN_VALUE);
            addDoubleToBuffer(strBuffer, LocationLibConst.KEY_LATITUDE, latitude);

            //経度
            double longitude = intent.getDoubleExtra(LocationLibConst.KEY_LONGITUDE, Double.MIN_VALUE);
            addDoubleToBuffer(strBuffer, LocationLibConst.KEY_LONGITUDE, longitude);

            //精度
            float accuracy = intent.getFloatExtra(LocationLibConst.KEY_ACCURACY, Float.MIN_VALUE);
            addFloatToBuffer(strBuffer, LocationLibConst.KEY_ACCURACY, accuracy);

            //標高
            double altitude = intent.getDoubleExtra(LocationLibConst.KEY_ALTITUDE, Double.MIN_VALUE);
            addDoubleToBuffer(strBuffer, LocationLibConst.KEY_ALTITUDE, altitude);

            //日付
            Date date;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                date = intent.getSerializableExtra(LocationLibConst.KEY_DATE, Date.class);
            } else {
                date = (Date) intent.getSerializableExtra(LocationLibConst.KEY_DATE);
            }

            if(date == null){
                date = new Date();
            }
            @SuppressLint("SimpleDateFormat")
            String dateString = new SimpleDateFormat("yyyyMMdd").format(date);
            addStringToBuffer(strBuffer, "Date", dateString);
            @SuppressLint("SimpleDateFormat")
            String timeString = new SimpleDateFormat("HHmmss").format(date);
            addStringToBuffer(strBuffer, "Time", timeString);

            //画面に受信結果を表示
            mLocationResultTextView.setText(strBuffer.toString());

            //非同期でジオフェンスライブラリの内外判定を実施
            checkArea(latitude, longitude, accuracy, date);
        }

        /**
         * 「測位結果（NGパターン）」通知（連続測位）
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedLocationNG(Intent intent) {
            Context ctx = SampleApplication.getAppContext();
            String cause = intent.getStringExtra(LocationLibConst.KEY_CAUSE);
            String messageFormat = ctx.getString(R.string.toast_location_lib_location_result_ng);
            String message = String.format(messageFormat, cause);
            Toast.makeText(LocationLibCheckActivity.this, message, Toast.LENGTH_LONG).show();
        }

        /**
         * 「認証エラー」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedAuthError(Intent intent) {

            Context ctx = SampleApplication.getAppContext();

            String messageString;
            int errorState = intent.getIntExtra(LocationLibConst.KEY_AUTH_ERROR_STATE, -1);
            switch(errorState) {
                case LocationLibConst.AUTH_ERROR_STATE_TOKEN_ISSUE:
                    //トークン発行認証失敗
                    messageString = ctx.getString(R.string.toast_location_lib_auth_error_state_token_issue);
                    break;
                case LocationLibConst.AUTH_ERROR_STATE_AUTH_NOT_AUTHORIZED:
                    //認証不許可
                    messageString = ctx.getString(R.string.toast_location_lib_auth_error_state_not_authorized);
                    break;
                default:
                    messageString = "ErrorState unknown";
                    break;
            }

            Toast.makeText(getBaseContext(), messageString, Toast.LENGTH_LONG).show();
        }

        /**
         * 「内部エラー」通知
         *
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedInternalError(Intent intent) {
            @StringRes int messageId;

            //「エラー内容」に応じて文字列を設定
            int errorState = intent.getIntExtra(LocationLibConst.KEY_ERROR_STATUS, -1);
            if (errorState == LocationLibConst.INTERNAL_ERROR_STATUS_NOT_START_LOCATION) {
                //予期せぬエラーによる測位停止
                messageId = R.string.toast_location_lib_internal_error_unexpected;
            } else {
                //未知のエラー
                messageId = R.string.toast_location_lib_internal_error_unknown;
            }

            Toast.makeText(getBaseContext(), messageId, Toast.LENGTH_LONG).show();
        }

        /**
         * 「オペレーション実行エラー」通知
         * @param intent 位置測位ライブラリからのIntent
         */
        public void onNotifiedOperationError(Intent intent) {
            Toast.makeText(LocationLibCheckActivity.this, R.string.toast_location_lib_operation_execution_error, Toast.LENGTH_LONG).show();
        }

        private void addStringToBuffer(StringBuffer strBuffer, String key, String value) {
            if (key != null && value != null) {
                strBuffer.append(key).append(" : ").append(value).append("\n");
            }
        }

        private void addFloatToBuffer(StringBuffer strBuffer, @SuppressWarnings("SameParameterValue") String key, Float value) {
            if (key != null && value != null && value != Float.MIN_VALUE) {
                strBuffer.append(key).append(" : ").append(value).append("\n");
            }
        }

        private void addDoubleToBuffer(StringBuffer strBuffer, String key, Double value) {
            if (key != null && value != null && value != Double.MIN_VALUE) {
                strBuffer.append(key).append(" : ").append(value).append("\n");
            }
        }
    };

    /**
     * 内外判定結果（例外発生）
     * @param messageResId メッセージID
     */
    void onError(@StringRes int messageResId) {
        Toast.makeText(getBaseContext(), messageResId, Toast.LENGTH_LONG).show();
    }

    /**
     * 内外判定結果（判定正常）
     * @param latitude 緯度
     * @param longitude 経度
     * @param accuracy 測定誤差
     * @param date 測位日時
     * @param checkResult 内外判定結果
     */
    void onResult(double latitude, double longitude, float accuracy, Date date, String checkResult) {
        //入力情報の整形
        @SuppressLint("SimpleDateFormat")
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
        //緯度、経度、誤差は小数点以下の有効値を全て出力（doubleのままで出力するとデフォルト6桁までしか表示されないのでStringで出力）
        String inputString = String.format(getString(R.string.location_lib_check_label_result_param),
                sdf.format(date), "" + latitude, "" + longitude, "" + accuracy);

        //判定結果を入力情報を結果エリアの先頭に追加表示
        String summaryText = checkResult + inputString + "\n" + mInOutResultTextView.getText();
        mInOutResultTextView.setText(summaryText);
    }
}
